#!/usr/bin/perl

# Usage: perl compareMethods.pl tokeep/theories/ tokeep/results/ results.csv

#takes two directories, one for theories, one for results. compares their files and outputs a tabulated file

use strict;
use warnings;
use Carp;
use scripts::Hyp2list;
  
my $theoryDir = shift;
my $resultsDir = shift;
my $outputFile = shift;

opendir(DIR, "$theoryDir") ||
    confess "couldn't open dir $theoryDir: $!. Stopped"; 
my @theoryFiles= sort sortTheory grep { $_ !~ /^\./}  readdir DIR;
closedir DIR;

open(FILE, ">$outputFile") ||
    confess "couldn't open file $outputFile: $!. Stopped"; 

#print header
print FILE "run\t";
foreach my $n ("", "_n"){
    foreach my $ex (100, 1000){
	foreach my $m ("Base", "Gen", "DP"){
	    print FILE "$m$ex$n\t";
	}
    }
}
print FILE "\n";

foreach my $theoryFile (@theoryFiles){
    next unless $theoryFile =~ /^run(\d+)_1.hyp$/;
    my $run = $1;
#    next if $run > 10;
    print FILE "$run\t";
    foreach my $n ("", "_n"){
	foreach my $ex (100, 1000){
#	    foreach my $m ("Base", "Gen", "DP"){
	    foreach my $m ("DP"){
		my $resultFile = "$resultsDir$m$run\_$ex$n.hyp";
		my $auc =  sprintf("%.3f",  Hyp2list::checkResults("$theoryDir$theoryFile", $resultFile) );
		print FILE "$auc\t";
	    }
	}
    }
    print FILE "\n";
}
closedir FILE;

#correctly sorts run10, run2
sub sortTheory{
    $a =~ /\D+(\d+)/;
    my $s1 = $1;
    $b =~ /\D+(\d+)/;
    my $s2 = $1;
    $s1 <=> $s2;
}
