%DP specific instructions
% Remember to set noise, minpos and m to 10% of each data cohort

%%% Aleph Parameters---------------------------------
:- set(clauselength,5).
:- set(evalfn,user). %Clause utility is -value returned by user-defined cost()
:- set(minpos,1).
:- set(m,1).
:- set(rulefile,'hyp.pl').
%:- set(noise,1).
%:- set(minacc,1).
%:- set(minscore,10).

%%% Train and Test Sets---------------------------------
:- set(record,true).
:- set(recordfile,'A10.trace').
%:- set(train_pos, 'trainsDP1.f').
%:- set(train_neg, 'trainsDP1.n').
:- set(train_pos, '../DPdata/trains1A10.f').
:- set(train_neg, '../DPdata/trains1A10.n').

%load alternate examples in different module, so they don't get mixed up
%:- [alt_train_pos:'trainsDP2.f'].
%:- [alt_train_neg:'trainsDP2.n'].
:- [alt_train_pos:'../DPdata/trains2A10.f'].
:- [alt_train_neg:'../DPdata/trains2A10.n'].


%%% Background Knowledge---------------------------------
%:- consult('trainsDP1.bk').
%:- consult('trainsDP2.bk').
:- consult('../DPdata/trains1A1000.bk').
:- consult('../DPdata/trains2A1000.bk').


%%% DP Functions---------------------------------
:- dynamic scoringFunction/1.
scoringFunction(coverage). %fact, global var. Default
mySet(scoringFunction,X) :-
        retractall(scoringFunction(_)), 
        %remove all occurances of scoringFunction
        assert(scoringFunction(X)). %and then place new ones

:- dynamic comparingFunction/1.
comparingFunction(subtraction). %fact, global var. Default
mySet(comparingFunction,X) :-
        retractall(comparingFunction(_)), 
        assert(comparingFunction(X)). %and then place new ones

compareScores(subtraction, A, B, Val) :-
        Val is (A-B).
compareScores(quotient, A, B, Val) :-
        B = 0 -> Val = inf ; Val is A/B. % ';' is "or"

:- dynamic alt_deleted/1. 
%attribute added to covered alternative positives deleted by induce


%%% Alternate Cost Function---------------------------------
%get the clause coverage on alternate pos and neg
apply_rule_on_alternate(Clause, AP, AN) :- 
        Clause = (H :- B), %specify clause shape
%       writeln(doing:Clause),get0(_), %get is waiting for a character input
        %get matches list
        findall( H, (alt_train_pos : H, \+ alt_deleted(H), once(B)), List_AP),
        length(List_AP, AP), %get the length of the list
        findall( H, (alt_train_neg : H, once(B)), List_AN),
        length(List_AN, AN).

%get total number of Positives and negatives
%P changes with induce because previously covered positives are removed
%total_pos(P) :-
%       '$aleph_global'(last_example,last_example(pos,P)).
total_pos(PC) :-
        '$aleph_global'(atoms_left,atoms_left(pos,Pos)),
        interval_count(Pos,PC).
total_neg(N) :-
        '$aleph_global'(last_example,last_example(neg,N)).

% total_apos(AP) :-
%       example(1,pos,H),%to get the head
%       predicate_property(alt_train_pos : H, number_of_clauses(AP)).
total_apos(APC) :-
        example(1,pos,H),%to get the head
        predicate_property(alt_train_pos : H, number_of_clauses(AP)),
        predicate_property(alt_deleted(_), number_of_clauses(APD)),
        APC is AP-APD.

total_aneg(AN) :-
        example(1,neg,H),
        predicate_property(alt_train_pos : H, number_of_clauses(AN)).

%called by aleph after induce, to remove covered Alternative Positives
remove_apos((H :- B)) :-
        alt_train_pos : H,
        \+ alt_deleted(H),
        once(B),
        assert(alt_deleted(H)),
        fail.
remove_apos(_).

%best clause is one with the smaller cost. That is why cost is usually negated
cost(Clause, [P, N, L], Cost) :-
        apply_rule_on_alternate(Clause, AP, AN),
%       writeln(scoring:P:N:AP:AN),
	evalfn(mestimate, [P,N,L], Acc1),
	evalfn(mestimate, [AP,AN,L], Acc2),	
	comparingFunction(ComparingF),
       	compareScores(ComparingF, Acc1, Acc2, Val1),
	Cost is -P * Val1.


%       (Acc1-Acc2 =:= 0 -> Cost is inf ; 
%		(scoringFunction(ScoringF),
%        	evalfn(ScoringF, [P,N,L], Cost1),
%        	evalfn(ScoringF, [AP,AN,L], Cost2),
%        	comparingFunction(ComparingF),
%        	compareScores(ComparingF, Cost1, Cost2, Val),
%        	Cost is -Val)
%	)
%       	,writeln(score:Cost)
%	.





%%% DP Parameters---------------------------------
%:- mySet(scoringFunction, wracc).
%:- mySet(comparingFunction, quotient).
:- mySet(scoringFunction, coverage).
:- mySet(comparingFunction, subtraction).

