
%%% Aleph Parameters---------------------------------
:- set(caching,true).
:- set(check_useless,true).
:- set(clauselength,4).
:- set(depth,100000).
:- set(i,10). 
:- set(nodes,50000).
:- set(optimise_clauses,true). %performs query optimisations
:- set(verbosity,0). %A value of 0 shows very little.
:- set(evalfn,user). %Clause utility is -value returned by user-defined cost()
:- set(minpos,10).
:- set(minacc,1).
:- set(samplesize,10).

%%% Train and Test Sets---------------------------------
:- set(record,true).
:- set(recordfile,'alternate.trace').
:- set(train_pos, 'trainsDP1.f').
:- set(train_neg, 'trainsDP1.n').

%load alternate examples in different module, so they don't get mixed up
:- [alt_train_pos:'trainsDP2.f'].
:- [alt_train_neg:'trainsDP2.n'].


%%% Mode Declarations ---------------------------------
:- modeh(1,east(+train)).

:- modeb(*,infront(+train,-car,-car)).
:-determination(east/1,infront/3).
:- modeb(*,has_car(+train,-car)).
:-determination(east/1,has_car/2).
:- modeb(1,long(+car)).
:-determination(east/1,long/1).
:- modeb(1,closed(+car)).
:-determination(east/1,closed/1).
:- modeb(1,short(+car)).
:-determination(east/1,short/1).
:- modeb(1,open(+car)).
:-determination(east/1,open/1).
:- modeb(1,double(+car)).
:-determination(east/1,double/1).
:- modeb(1,jagged(+car)).
:-determination(east/1,jagged/1).
:- modeb(1,load(+car,#shape,#int)).
:-determination(east/1,load/3).
:- modeb(1,wheels(+car,#int)).
:-determination(east/1,wheels/2).


%%% Additional predicates ---------------------------------
shape(circle).  shape(ellipse).  shape(hexagon).  shape(rectangle).
shape(triangle).  shape(utriangle).  shape(u_shaped). shape(diamond).


%%% Background Knowledge---------------------------------
:- consult('trainsDP1.bk').
:- consult('trainsDP2.bk').


%%% DP Functions---------------------------------
:- dynamic scoringFunction/1.
scoringFunction(coverage). %fact, global var. Default
mySet(scoringFunction,X) :-
        retractall(scoringFunction(_)), 
        %remove all occurances of scoringFunction
        assert(scoringFunction(X)). %and then place new ones

:- dynamic comparingFunction/1.
comparingFunction(subtraction). %fact, global var. Default
mySet(comparingFunction,X) :-
        retractall(comparingFunction(_)), 
        assert(comparingFunction(X)). %and then place new ones

compareScores(subtraction, A, B, Val) :-
        Val is (A-B).
compareScores(quotient, A, B, Val) :-
        B = 0 -> Val = inf ; Val is A/B. % ';' is "or"

:- dynamic alt_deleted/1. 
%attribute added to covered alternative positives deleted by induce


%%% Alternate Cost Function---------------------------------
%get the clause coverage on alternate pos and neg
apply_rule_on_alternate(Clause, AP, AN) :- 
        Clause = (H :- B), %specify clause shape
%       writeln(doing:Clause),get0(_), %get is waiting for a character input
        %get matches list
        findall( H, (alt_train_pos : H, \+ alt_deleted(H), once(B)), List_AP),
        length(List_AP, AP), %get the length of the list
        findall( H, (alt_train_neg : H, once(B)), List_AN),
        length(List_AN, AN).

%get total number of Positives and negatives
%P changes with induce because previously covered positives are removed
%total_pos(P) :-
%       '$aleph_global'(last_example,last_example(pos,P)).
total_pos(PC) :-
        '$aleph_global'(atoms_left,atoms_left(pos,Pos)),
        interval_count(Pos,PC).
total_neg(N) :-
        '$aleph_global'(last_example,last_example(neg,N)).

% total_apos(AP) :-
%       example(1,pos,H),%to get the head
%       predicate_property(alt_train_pos : H, number_of_clauses(AP)).
total_apos(APC) :-
        example(1,pos,H),%to get the head
        predicate_property(alt_train_pos : H, number_of_clauses(AP)),
        predicate_property(alt_deleted(_), number_of_clauses(APD)),
        APC is AP-APD.

total_aneg(AN) :-
        example(1,neg,H),
        predicate_property(alt_train_pos : H, number_of_clauses(AN)).

%called by aleph after induce, to remove covered Alternative Positives
remove_apos((H :- B)) :-
        alt_train_pos : H,
        \+ alt_deleted(H),
        once(B),
        assert(alt_deleted(H)),
        fail.
remove_apos(_).

%best clause is one with the smaller cost. That is why cost is usually negated
cost(Clause, [P, N, L], Cost) :-
        apply_rule_on_alternate(Clause, AP, AN),
%       writeln(scoring:P:N:AP:AN),

	evalfn(accuracy, [P,N,L], Acc1),
	evalfn(accuracy, [AP,AN,L], Acc2),
	
%        Acc1-Acc2 =:= 0 -> Cost is -0 ; 

        scoringFunction(ScoringF),
        evalfn(ScoringF, [P,N,L], Cost1),
        evalfn(ScoringF, [AP,AN,L], Cost2),
        comparingFunction(ComparingF),
        compareScores(ComparingF, Cost1, Cost2, Val),
        Cost is -Val.
%       writeln(score:Cost).

% ( a -> b, c ; e,f).

%%% DP Parameters---------------------------------
%:- mySet(scoringFunction, wracc).
%:- mySet(comparingFunction, quotient).
:- mySet(scoringFunction, accuracy).
:- mySet(comparingFunction, subtraction).

