#!/u/vitor/bin/yap -L --
#!/usr/bin/yap -L --
#


:- initialization(main).

%% :- ensure_loaded(rtrain).

parameter(rsize, [0.4,0.6]).
parameter(cases, [0.6,0.2,0.2]).
parameter(linked, [0.5,0.5]).
parameter(attribute, [0.15,0.15,0.15,0.4,0.15]).
parameter(load, [0.4,0.4,0.2]).


main :-
	unix(argv(L)),
	generate_theories(L, 1, 1).

generate_theories([], _, _).
generate_theories(N.L, I, T0) :-
	atom_number(N, NA),
	format('%%%%%%%%%%% THEORY ~d~n',[I]),
	generate_rules(NA),
	I1 is I+1,
        T1 is T0+NA,
	generate_theories(L, I1, T1).

generate_rules(0) :- !.
generate_rules(I0) :- !,
	generate_rule(Rule),
	portray_clause(Rule),
	I is I0-1,
	generate_rules(I).

generate_rule((east(T) :- Rule)) :-
	do_generate_rule(T, Rule).

do_generate_rule(T, ActualRule) :-
	items(Goals, _Size),
	connect(Goals, T, _, NewGoals),
	attributes(NewGoals, _, Attributes),
	build_rule(Attributes, Rule),
	!,
     	conclude_rule(T, Rule, ActualRule).
do_generate_rule(_, _) :-
throw(error(failed)).


conclude_rule(T, Rule, Rule) :-
	recordzifnot(rules, Rule, _), !.
conclude_rule(T, _, NR) :-
	do_generate_rule(T, NR).

build_rule([A], A) :- !.
build_rule(A.Attributes, (A,R)) :-
	build_rule(Attributes, R).


items(Goals, Size) :-
	parameter(rsize, Dist),
	pick(1, Dist, Size),
	length(Goals, Size).

connect([], _, _, []).
connect([G1,G2|Goals], T, C1, [infront(T,C1,C2)|NewGoals]) :-
	link, !,
	connect(Goals, T, _, NewGoals).
connect([_|Goals], T, C1, [has_car(T,C1)|NewGoals]) :-
	connect(Goals, T, _, NewGoals).

link :-
	parameter(cases, D),
	pick(1, D, 2).

attributes([], _, []).
attributes([infront(T, C1, C2)|Cars], C11, [infront(T, C1, C2),A1,A2|NewClause]) :-
	C11 \== C1, !,
	gen_attribute(C1, A1),
	gen_attribute(C2, A2),
	attributes(Cars, C2, NewClause).
attributes([infront(T, C1, C2)|Cars], _, [infront(T, C1, C2),A2|NewClause]) :-
	gen_attribute(C2, A2),
	attributes(Cars, C2, NewClause).
attributes([has_car(T, C)|Cars], _, [has_car(T, C),A|NewClause]) :-
	gen_attribute(C, A),
	attributes(Cars, C2, NewClause).

gen_attribute(C, A) :-
	parameter(attribute, D),
	pick(1, D, I),
	generate_case(I, C, A).

generate_case(1, C, double(C)).
generate_case(2, C, A) :-
	pick_uniform(2, Shape),
	translate_roof(Shape, C, A).
generate_case(3, C, A) :-
	Wheel is 3, %force it to just 3, since most are 2. trains are only 2-3
	translate_wheel(Wheel, C, A). 
generate_case(4, C, A) :-
	parameter(load,LD),
	pick(1, LD, Shapes),
	pick_uniform(8, Shape),
	translate_shape(Shape, C, Shapes, A).
generate_case(5, C, jagged(C)).


translate_shape(1, C, Shapes, load(C, circle, Shapes)).
translate_shape(2, C, Shapes, load(C, ellipse, Shapes)).
translate_shape(3, C, Shapes, load(C, hexagon, Shapes)).
translate_shape(4, C, Shapes, load(C, rectangle, Shapes)).
translate_shape(5, C, Shapes, load(C, triangle, Shapes)).
translate_shape(6, C, Shapes, load(C, utriangle, Shapes)).
translate_shape(7, C, Shapes, load(C, u_shaped, Shapes)).
translate_shape(8, C, Shapes, load(C, diamond, Shapes)).

translate_length(1, C, short(C)).
translate_length(2, C, long(C)).

translate_roof(1, C, open(C)).
translate_roof(2, C, closed(C)).
%% translate_roof(3, C, none(C)).
%% translate_roof(4, C, flat(C)).
%% translate_roof(5, C, jagged(C)).
%% translate_roof(6, C, peacked(C)).
%% translate_roof(7, C, arc(C)).

translate_wheel(1, C, wheels(C,1)).
translate_wheel(2, C, wheels(C,2)).
translate_wheel(3, C, wheels(C,3)).
translate_wheel(4, C, wheels(C,4)).

pick(I,Dist,N) :-
	X is random,
	pick(I,X,Dist,N).

pick(I0, X, X0._ , I0) :- X =< X0, !.
pick(I0, X, X0.Dist, N) :-
	I1 is I0+1,
	NX is X-X0,
	pick(I1,NX,Dist,N).

pick_uniform(Max, N) :-
	N is integer(random*Max)+1.

