# Usage: perl createB.pl -m DP -r 2 -n 100 -l 4 -noise 0
# perl createB.pl -m "$method" -r "$RUN" -n "$nb" -l "$CLAUSELENGTH" -noise "$noise"

use strict;
use warnings;
use Carp;

#Process input and create files
my %args = @ARGV;
my $method  = $args{-m} || confess "Input a method as: -m method[DP|Gen|Base]";
my $run = $args{-r} || confess "Input the run number as: -r run";
my $nb = $args{-n} || confess "Input the instance number per cohort as: -n nb";
my $clauseLength = $args{-l} || confess "Input the max clause length as: -l length";
my $noise = $args{-noise} || 0; #sets noise

#create aleph parameters
my $data; #because Gen method operates on DP data
($method eq "Gen")?
    ($data = "../data/DP") :
    ($data = "../data/".$method);
my $results = "../results/";
my $output = "instructions.b";
my $input = $method.$output;
my $tenPercent = $nb / 10; #set various aleph settings to 10% of data
my $noisy = "";
$noisy = "_n" if $noise; #add the last _n for noisy data

open(INPUT, $input) ||
    confess "couldn't open ", $input, ": $!. Stopped"; 
open(OUTPUT, '>', "instructions.b") ||
    confess "couldn't open ", $output, ": $!. Stopped";

#write aleph .b file
while (my $line = <INPUT>){
    $line =~ s/(set\(clauselength,)(\s*\d*)/$1$clauseLength/;
    $line =~ s/(set\((minpos|m|noise),)(\s*\d*)/$1$tenPercent/;
    $line =~ s/(set\(recordfile,\s*)(.*)(\).)/$1'$results$method$run\_$nb$noisy.trace'$3/;
    $line =~ s/(set\(rulefile,\s*)(.*)(\).)/$1'$results$method$run\_$nb$noisy.hyp'$3/;

    $line =~ s/(set\(train_pos,\s*)(.*)(\).)/$1'$data$run\_1_$nb$noisy.f'$3/;
    $line =~ s/(set\(train_neg,\s*)(.*)(\).)/$1'$data$run\_1_$nb$noisy.n'$3/;
    $line =~ s/(alt_train_pos:\s*)(.*)(\].)/$1'$data$run\_2_$nb$noisy.f'$3/;
    $line =~ s/(alt_train_neg:\s*)(.*)(\].)/$1'$data$run\_2_$nb$noisy.n'$3/;
    $line =~ s/(consult\(.*?)(\d+)/$1$run/;

    #if allowing noise or not
    ($noise)?
	($line =~ s/^(\%+)(.*set\(noise)/$2/) :
	($line =~ s/(^.*set\(noise)/\%$1/);

    print OUTPUT $line;
}
