#!/usr/bin/perl

# it checks if a true and a predicted theories are equivalent. It considers a large number of TN, and computes AUC-PR

#Usage: perl scripts/Hyp2list.pm theory1.pl runs/results/Gen1_100_n.hyp 
# Calls variant.yap, instead of checkRules


package Hyp2list;

use strict;
use warnings;
use Carp;
use List::Util qw[min sum];

#printf "%.3f", &checkResults(@ARGV), "\n";


sub checkResults{
    #get the theories
    my $theoryFile = shift;
    my $resultFile = shift;
    my $listFile = "output.temp";

    my $trueDP = &read_theory($theoryFile); #keys are rules, values are 1
    my $predDP = &read_result($resultFile); #keys are rules, values are scores
    my $maxRecall = &matchRules($listFile, $trueDP, $predDP);
    my $result = &printAUCPR($maxRecall, $listFile);
    return  $result;
}

#performs AUC-PR and prints it
# computes recall for 
sub printAUCPR(){
    my $maxRecall = shift;
    my $listFile = shift;
    my $temp = "temp.temp";
    system("java -jar ~/bin/auc_2.0.jar -t list $listFile > $temp");
    open(TEMP, "$temp") ||
	confess "couldn't open file $temp: $!. Stopped"; 
    <TEMP> =~ /Recall is (.*)$/; #first line
    my $totRecall = $1;
    close(TEMP);
    system("java -jar ~/bin/auc_2.0.jar -t list $listFile -r $maxRecall > $temp");
    open (TEMP, "$temp") ||
	confess "couldn't open file $temp: $!. Stopped"; 
    <TEMP> =~ /Recall is (.*)$/; #first line
    my $minRecall = $1;
    close(TEMP);
    system("rm $listFile $temp");
    return $totRecall-$minRecall;
}

#performs AUC-PR and prints it
sub printAUCPR_old(){
    my $maxRecall = shift;
    my $temp = "temp.temp";
    my $listFile = shift;
    system("java -jar ~/bin/auc_2.0.jar -t list $listFile > $temp");
    open(TEMP, "$temp") ||
	confess "couldn't open file $temp: $!. Stopped"; 
    <TEMP> =~ /Recall is (.*)$/; #first line
    my $result = $1;
    close(TEMP);
    system("rm $listFile $temp");
    return $result;
}

#match the pred and theory rules, create a list file
#returns the recall of the induced thoery
sub matchRules(){
    #consider TN is 50000, and FN are the last ones
    my $TN = 50000; #for old method. Not used

    my $listFile = shift;
    my $trueDP = shift;
    my $predDP = shift;
    open(FILE, '>', $listFile) ||
	confess "couldn't open ", $listFile, ": $!. Stopped"; 

    #check the class of the predicted rules
    foreach my $rule (keys %{$predDP}){
	my $result = 0;
	foreach my $target (keys %{$trueDP} ){
	    my $x = `./scripts/variant.yap \"$rule\" \"$target\"`;
	    chomp $x;
	    $result = int($x);
#	    $result = &checkRules($rule, $target);
	    if ($result) {
		$trueDP->{$target} = 0;
		last;
	    }
	}
	($result) ?
	    (print FILE $predDP->{$rule}, "\ttrue\n") :
	    (print FILE $predDP->{$rule}, "\tfalse\n") ;
    }
    
    #add one additional negative rule at the end (better suited for AUC computation)
    my $minScore = min(values %{$predDP}) || 0; #if no rules returned
    print FILE $minScore-1, "\tfalse\n"; #adds 1 TN. should perform recall substraction    
    #if we have FN
    my $FN = sum(values %{$trueDP});
    if ($FN){
#	print FILE $minScore-1, "\tfalse\n" foreach (1..$TN);
	print FILE $minScore-2, "\ttrue\n" foreach (1..$FN);
    }
    close (FILE);
    return (scalar(keys %{$trueDP})-$FN)/ scalar(keys %{$trueDP});
}

# Given result file, extracts predicted DP rules
sub read_result(){
    my $file = shift;
    my %predictedDP;
    open(FILE, $file) ||
	confess "couldn't open ", $file, ": $!. Stopped"; 

    my $clause;
    while (my $line = <FILE>){
	chomp $line;
	($line =~ /cost:(.*)$/) ?
	    ($predictedDP{$clause} = $1) :
	    ($clause = $line);
    }

    #prune if Base
    my %keep;
    if ($file =~ /Base/){
	foreach my $k (keys %predictedDP){
	    if ($k =~  /category\(.*?,1\)/){
		my $v = $predictedDP{$k};
		$k =~ s/(,\s*)?category\(.*?,1\)//;
		$keep{$k} = $v;
	    }
	}
	return \%keep;
    } 
    return \%predictedDP;
}


# Given theory file, extracts true DP rules
sub read_theory(){
    my $file = shift;
    my %trueDP;
    
    open(FILE, $file) ||
	confess "couldn't open ", $file, ": $!. Stopped"; 

    my $diffTriger = 0; #consider rules AFTER different
    while (my $line = <FILE>){
	chomp $line;
	if ($line =~ /Different/){
	    $diffTriger = 1;
	    next;
	}
	next unless $diffTriger ; 
	$trueDP{$line} = 1;
    }
    return \%trueDP;
}

# Given two rules, checks if they are the same. Returns 0/1
sub checkRules(){
    my $l1 = shift;
    my $l2 = shift;

    #remove all spaces
    $l1 =~ s/\s//g;
    $l2 =~ s/\s//g;

    #count predicates
    my ($H1, $B1) = split(/:-/,$l1);
    my ($H2, $B2) = split(/:-/,$l2);
    my @body1 = split(/(?<=\)),/,$B1);
    my @body2 = split(/(?<=\)),/,$B2);
    return 0 if $#body1 != $#body2; #stop if different body term count
#    print "Same body count\n";

    my %usedVars;
    #first compare heads
    return 0 if ! &unifyHead($H1,$H2,\%usedVars); #stop if head didn't unify
#    print "head unified\n";

    #for each predicate of rule1, unify
    foreach my $term1 (@body1){
#	print "Term1 is: $term1\n";
	return 0 if ! &unifyBody($term1,\@body2,\%usedVars);
#	print "$term1 unified\n";
    }
    return 1;
}


sub unifyBody(){
    my $term1 = shift;
    my $clause2 = shift;
    my $usedVars = shift;
    my ($p1, $v1) = ($term1  =~ /(\w+)\((.*?)\)/);
    
    #check each of the second clause terms
    foreach my $t (0..$#{$clause2}){
	my ($p2, $v2) = ($clause2->[$t]  =~ /(\w+)\((.*?)\)/);
#	print "Matching it with ", $clause2->[$t], "\n";
	next if ($p1 ne $p2); #not same predicate
#	print "same predicate\n";

	my @vars1 = split(/,/, $v1);
	my @vars2 = split(/,/, $v2);
	next if ($#vars1 != $#vars2); #not same predicate signature
#	print "same signature\n";

	my %tempVars;
	foreach my $i (0..$#vars1){
#	    print "checking: ", $vars1[$i], ", ", $vars2[$i], "\n"; 
	    my $inc1 = ($vars1[$i] =~ /\b([A-Z]\w*|_\w+)\b/); #Variables start with an upper case, or _
	    my $inc2 = ($vars2[$i] =~ /\b([A-Z]\w*|_\w+)\b/);
	    last if $inc1 != $inc2; #one is a variable, the other not
#	    print "both same type\n";

	    #make sure non-variables are the same
	    if ($inc1 == 0){
		next if $vars1[$i] eq $vars2[$i];
		return 0; #different non-variables
	    }
#	    print "are variables\n";
	    $tempVars{$vars1[$i]} = $vars2[$i]; #unify the variables
	}

	#check if correct unified variables
	my $ok = 0;
	foreach my $k (keys %{$usedVars}){
	    $ok = 1 if exists($tempVars{$k}) && $usedVars->{$k} eq $tempVars{$k};
	}
	next if !$ok; #correct term, different binding
	
	$usedVars->{$_} = $tempVars{$_} foreach (keys %tempVars);	
	splice(@{$clause2},$t,1);
	return 1;
    }
    return 0;
}


sub unifyHead(){
    my $term1 = shift;
    my $term2 = shift;
    my $usedVars = shift;

    my ($p1, $v1) = ($term1  =~ /(\w+)\((.*?)\)/);
    my ($p2, $v2) = ($term2  =~ /(\w+)\((.*?)\)/);
    return 0 if ($p1 ne $p2); #not same predicate

    my @vars1 = split(/,/, $v1);
    my @vars2 = split(/,/, $v2);
    return 0 if ($#vars1 != $#vars2); #not same predicate signature
    
    foreach my $i (0..$#vars1){
	my $inc1 = ($vars1[$i] =~ /\b([A-Z]\w*|_\w+)\b/);
	my $inc2 = ($vars2[$i] =~ /\b([A-Z]\w*|_\w+)\b/);
	return 0 if $inc1 != $inc2; #one is a variable, the other not

	#make sure non-variables are the same
	if ($inc1 == 0){
	    next if $vars1[$i] eq $vars2[$i];
	    return 0; #different non-variables
	}
	$usedVars->{$vars1[$i]} = $vars2[$i]; #unify the variables
    }
    return 1;
}

1; 

